/***************************************************************************//**
* \file main_cm4_PWM_ADC.c
* \version 1.0
*
* Demonstrates ADC triggered by PWM.
*
********************************************************************************
* \copyright
* Copyright 2020, Cypress Semiconductor Corporation. All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*
*******************************************************************************/

/******************************************************************************/
/* Include Files                                                              */
/******************************************************************************/
#include <stdio.h>
#include <stdarg.h>
#include "cy_project.h"
#include "cy_device_headers.h"

/******************************************************************************/
/* Macro Definitions                                                          */
/******************************************************************************/
#define DIV_ROUND_UP(a,b) (((a) + (b)/2) / (b))

#define periFreq                        (80000000ul)
#define targetFreq                       (2000000ul)
#define divNum                                (40ul) // periFreq div targetFreq

/* PWM Mode Configuration def */
#define TCPWMx_GRPx_CNTx_PWM            TCPWM0_GRP1_CNT0
#define PCLK_TCPWMx_CLOCKSx_PWM         PCLK_TCPWM0_CLOCKS256
#define TCPWM_PERI_CLK_DIVIDER_NO_PWM   1ul

#define TCPWMx_PWM_PRESCALAR_DIV_x      CY_TCPWM_PWM_PRESCALER_DIVBY_128  // 2,000,000 / 128 = 15625 Hz
#define TCPWMx_PERIOD                   0x4000ul    // 15625 Hz / 16384 (0x4000) = 0,95Hz (PWM frequency)
#define TCPWMx_COMPARE0                 0x1000ul    // 0x1000 / 0x4000 = 0.25 (PWM duty)

#define BB_POTI_ANALOG_PCLK             PCLK_PASS0_CLOCK_SAR0
#define ADC_LOGICAL_CHANNEL             0ul
#define ADC_OPERATION_FREQUENCY_MAX_IN_HZ  (26666666ul)
#define ANALOG_IN_SAMPLING_TIME_MIN_IN_NS  (1000ull)

/******************************************************************************/
/* Function Prototypes                                                        */
/******************************************************************************/
void MyPWM_IntrISR(void);
void AdcIntHandler(void);

/******************************************************************************/
/* Global Variables                                                           */
/******************************************************************************/
uint16_t               resultBuff[16];
cy_stc_adc_ch_status_t statusBuff[16];
uint8_t                resultIdx;
/*******************************************************************************
* Function Name: main                                                    
********************************************************************************
*
* \brief 
*
* \param 
* None.
*
* \return
* int
*
* \note 
* None.
*******************************************************************************/

int main(void)
{
    SystemInit();
    
    /** Enable global interrupts. */
    __enable_irq();

    /** Initialize UART, Port, and TCPWM */
    Cy_Semihosting_InitAll(CY_USB_SCB_TYPE, 115200ul, NULL, false);
    printf("Initialized Semihosting Serial I/O.\n\r");

    /** ADC Initalisation, Input = Poti, Conversion triggered by TCPWM */
    
    /** PORT init for driving of LED4 = LED0 according to bb_bsp_tviibe1m_starter_kit.h */
    cy_stc_gpio_pin_config_t pin_config = {
      .outVal       = CY_GPIO_ZERO,                   /**< Pin output state */
      .driveMode    = CY_GPIO_DM_STRONG_IN_OFF,       /**< Drive mode */
      .hsiom        = CY_LED0_PIN_MUX,                /**< HSIOM selection */
      .intEdge      = CY_GPIO_INTR_DISABLE,           /**< Interrupt Edge type */
      .intMask      = CY_GPIO_ZERO,                   /**< Interrupt enable mask */
      .vtrip        = CY_GPIO_VTRIP_CMOS,             /**< Input buffer voltage trip type */
      .slewRate     = CY_GPIO_SLEW_SLOW,              /**< Output buffer slew rate */
      .driveSel     = CY_GPIO_DRIVE_STRENGTH_DEFAULT, /**< Drive strength */
    };
    Cy_GPIO_Pin_Init(CY_LED0_PORT, CY_LED0_PIN, &pin_config);
    
    /** MyTCPWM Initialisation */
    cy_stc_tcpwm_pwm_config_t const MyPWM_config ={
      .pwmMode            = CY_TCPWM_PWM_MODE_PWM,
      .clockPrescaler     = TCPWMx_PWM_PRESCALAR_DIV_x,
      .debug_pause        = false,
      .Cc0MatchMode       = CY_TCPWM_PWM_TR_CTRL2_CLEAR,
      .OverflowMode       = CY_TCPWM_PWM_TR_CTRL2_SET,
      .UnderflowMode      = CY_TCPWM_PWM_TR_CTRL2_NO_CHANGE,
      .Cc1MatchMode       = CY_TCPWM_PWM_TR_CTRL2_NO_CHANGE,
      .deadTime           = 0ul,
      .deadTimeComp       = 0ul,
      .runMode            = CY_TCPWM_PWM_CONTINUOUS,
      .period             = TCPWMx_PERIOD - 1ul,
      .period_buff        = 0ul,
      .enablePeriodSwap   = false,
      .compare0           = TCPWMx_COMPARE0,
      .compare1           = 0ul,
      .enableCompare0Swap = false,
      .enableCompare1Swap = false,
      .interruptSources   = 3ul,  // Demask CC0 and TC interrupt requests
      .invertPWMOut       = 0ul,
      .invertPWMOutN      = 0ul,
      .killMode           = CY_TCPWM_PWM_STOP_ON_KILL,
      .switchInputMode    = 3ul,
      .switchInput        = 0ul,
      .reloadInputMode    = 3ul,
      .reloadInput        = 0ul,
      .startInputMode     = 3ul,
      .startInput         = 0ul,
      .kill0InputMode     = 3ul,
      .kill0Input         = 0ul,
      .kill1InputMode     = 3ul,
      .kill1Input         = 0ul,
      .countInputMode     = 3ul,
      .countInput         = 1ul,   
    };

    /* Assign a programmable divider  for TCPWMx_GRPx_CNTx_PWM */
    Cy_SysClk_PeriphAssignDivider(PCLK_TCPWMx_CLOCKSx_PWM, CY_SYSCLK_DIV_16_BIT, TCPWM_PERI_CLK_DIVIDER_NO_PWM);
    /* Sets the 16-bit divider */
    Cy_SysClk_PeriphSetDivider(CY_SYSCLK_DIV_16_BIT, TCPWM_PERI_CLK_DIVIDER_NO_PWM, (divNum-1ul));
    /* Enable the divider */
    Cy_SysClk_PeriphEnableDivider(CY_SYSCLK_DIV_16_BIT, TCPWM_PERI_CLK_DIVIDER_NO_PWM);
    
    /** Configure TCPWM-Interrupt */
    cy_stc_sysint_irq_t         stc_sysint_irq_cfg_mytcpwm;
    stc_sysint_irq_cfg_mytcpwm.sysIntSrc = tcpwm_0_interrupts_256_IRQn; // Benenne System-Interrupt-Nummer, oder System-Interrupt-Source Nr., hier = scb_0_interrupt_IRQn = 17
    stc_sysint_irq_cfg_mytcpwm.intIdx    = CPUIntIdx3_IRQn;      // Benenne CPU-Interrupt-Nummer oder User-Interrupt-Nummer, hier 2
    stc_sysint_irq_cfg_mytcpwm.isEnabled = true;                 // Enabled den System-Interrupt mit Nummer sysIntScr fr diese CPU
    Cy_SysInt_SetSystemIrqVector(stc_sysint_irq_cfg_mytcpwm.sysIntSrc, MyPWM_IntrISR); // Zuordnung User-ISR namens Scb_UART_IntrISR zu System-IRQ mit Nummer sysIntSrc
    Cy_SysInt_InitIRQ(&stc_sysint_irq_cfg_mytcpwm);              // Ausfhrung der Zuordnung
 
    // Define CPU-Interrupt Priority and unmask interrupt request path
    NVIC_SetPriority(stc_sysint_irq_cfg_mytcpwm.intIdx, 3ul);    // niedrigste Prio = 7, hchste Prio = 0
    Cy_Tcpwm_Counter_SetCC0_IntrMask(TCPWMx_GRPx_CNTx_PWM);
    Cy_Tcpwm_Counter_Enable(TCPWMx_GRPx_CNTx_PWM);
    Cy_Tcpwm_TriggerStart(TCPWMx_GRPx_CNTx_PWM);

    /* Initialize TCPWM0_GRPx_CNTx_PWM_PR as PWM Mode & Enable */
    Cy_Tcpwm_Pwm_Init(TCPWMx_GRPx_CNTx_PWM, &MyPWM_config);
    Cy_Tcpwm_Pwm_Enable(TCPWMx_GRPx_CNTx_PWM);
    Cy_Tcpwm_TriggerStart(TCPWMx_GRPx_CNTx_PWM);

    /** Trigger Multiplexer */
    Cy_TrigMux_Connect1To1(TRIG_IN_1TO1_1_TCPWM_TO_PASS_CH_TR0, CY_TR_MUX_TR_INV_DISABLE, TRIGGER_TYPE_LEVEL, 0);

    printf("Initialized TCPWM.\n\r");

    /** Initialize ADC */
    /** ADC Port Init */
    cy_stc_gpio_pin_config_t adcPinConfig =
        {
            .outVal    = 0ul,
            .driveMode = CY_GPIO_DM_ANALOG,
            .hsiom     = CY_ADC_POT_PIN_MUX,
            .intEdge   = 0ul,
            .intMask   = 0ul,
            .vtrip     = 0ul,
            .slewRate  = 0ul,
            .driveSel  = 0ul,
        };
     Cy_GPIO_Pin_Init(CY_ADC_POT_PORT, CY_ADC_POT_PIN, &adcPinConfig);

     /** ADC Clock Config */
     uint32_t actualAdcOperationFreq;
     uint32_t divNumSAR0 = DIV_ROUND_UP(periFreq, ADC_OPERATION_FREQUENCY_MAX_IN_HZ);
     
     actualAdcOperationFreq = periFreq / divNumSAR0;
     
     Cy_SysClk_PeriphAssignDivider(BB_POTI_ANALOG_PCLK, CY_SYSCLK_DIV_16_BIT, 0ul);
     Cy_SysClk_PeriphSetDivider(CY_SYSCLK_DIV_16_BIT, 0ul, (divNumSAR0 - 1ul));
     Cy_SysClk_PeriphEnableDivider(CY_SYSCLK_DIV_16_BIT, 0ul);
     
     uint32_t samplingCycle = (uint32_t)DIV_ROUND_UP((ANALOG_IN_SAMPLING_TIME_MIN_IN_NS * (uint64_t)actualAdcOperationFreq), (1000000000ul));

     cy_stc_adc_config_t adcConfig =
        {
            .preconditionTime    = 0u,
            .powerupTime         = 0u,
            .enableIdlePowerDown = false,
            .msbStretchMode      = CY_ADC_MSB_STRETCH_MODE_1CYCLE,
            .enableHalfLsbConv   = 0u,
            .sarMuxEnable        = true,
            .adcEnable           = true,
            .sarIpEnable         = true,
        };
     
     cy_stc_adc_channel_config_t adcChannelConfig =
        {
            .triggerSelection          = CY_ADC_TRIGGER_TCPWM,
            .channelPriority           = 0u,
            .preenptionType            = CY_ADC_PREEMPTION_FINISH_RESUME,
            .isGroupEnd                = true,
            .doneLevel                 = CY_ADC_DONE_LEVEL_PULSE,
            .pinAddress                = (cy_en_adc_pin_address_t) CY_ADC_POT_IN_NO,
            .portAddress               = CY_ADC_PORT_ADDRESS_SARMUX0,
            .extMuxSelect              = 0u,
            .extMuxEnable              = false,
            .preconditionMode          = CY_ADC_PRECONDITION_MODE_OFF,
            .overlapDiagMode           = CY_ADC_OVERLAP_DIAG_MODE_OFF,
            .sampleTime                = samplingCycle,
            .calibrationValueSelect    = CY_ADC_CALIBRATION_VALUE_REGULAR,
            .postProcessingMode        = CY_ADC_POST_PROCESSING_MODE_NONE,
            .resultAlignment           = CY_ADC_RESULT_ALIGNMENT_RIGHT,
            .signExtention             = CY_ADC_SIGN_EXTENTION_UNSIGNED,
            .averageCount              = 0u,
            .rightShift                = 0u,
            .rangeDetectionMode        = CY_ADC_RANGE_DETECTION_MODE_INSIDE_RANGE,
            .rangeDetectionLoThreshold = 0x0000u,
            .rangeDetectionHiThreshold = 0x0FFFu,
            .mask.grpDone              = true,
            .mask.grpCancelled         = false,
            .mask.grpOverflow          = false,
            .mask.chRange              = false,
            .mask.chPulse              = false,
            .mask.chOverflow           = false,
        };

    Cy_Adc_Init(CY_ADC_POT_MACRO, &adcConfig);
    Cy_Adc_Channel_Init(&CY_ADC_POT_MACRO->CH[ADC_LOGICAL_CHANNEL], &adcChannelConfig);

    /* Register ADC interrupt handler and enable interrupt */
    cy_stc_sysint_irq_t irq_cfg;
    irq_cfg = (cy_stc_sysint_irq_t){
            .sysIntSrc  = (cy_en_intr_t)((uint32_t)CY_ADC_POT_IRQN + ADC_LOGICAL_CHANNEL),
            .intIdx     = CPUIntIdx3_IRQn,
            .isEnabled  = true,
        };
    Cy_SysInt_InitIRQ(&irq_cfg);
    Cy_SysInt_SetSystemIrqVector(irq_cfg.sysIntSrc, AdcIntHandler);
    NVIC_SetPriority(irq_cfg.intIdx, 3ul);
    NVIC_EnableIRQ(irq_cfg.intIdx);
    
    /* Enable ADC ch. */
    Cy_Adc_Channel_Enable(&CY_ADC_POT_MACRO->CH[ADC_LOGICAL_CHANNEL]);
    
    while (true)
    { 

    }   
} // of main()

void MyPWM_IntrISR(void)
{
   if(Cy_Tcpwm_Counter_GetCC0_IntrMasked(TCPWMx_GRPx_CNTx_PWM))
   {
    // CCO capture
    Cy_Tcpwm_Counter_ClearCC0_Intr(TCPWMx_GRPx_CNTx_PWM); // Clear Interrupt Flag
    Cy_GPIO_Clr(CY_LED0_PORT, CY_LED0_PIN);  // Toggle blue LED
   }
   
   if(Cy_Tcpwm_Counter_GetTC_IntrMasked(TCPWMx_GRPx_CNTx_PWM))
   {
    // TC capture
    Cy_Tcpwm_Counter_ClearTC_Intr(TCPWMx_GRPx_CNTx_PWM); // Clear Interrupt Flag
    Cy_GPIO_Set(CY_LED0_PORT, CY_LED0_PIN);  // Toggle blue LED
   }   
}

void AdcIntHandler(void)
{
    cy_stc_adc_interrupt_source_t intrSource = { false };

    /* Get interrupt source */
    Cy_Adc_Channel_GetInterruptMaskedStatus(&CY_ADC_POT_MACRO->CH[ADC_LOGICAL_CHANNEL], &intrSource);

    if(intrSource.grpDone)
    {
        /* Get the result(s) */
        Cy_Adc_Channel_GetResult(&CY_ADC_POT_MACRO->CH[ADC_LOGICAL_CHANNEL], &resultBuff[resultIdx], &statusBuff[resultIdx]);

        /* Display ADC result */
        printf("ADC result = %d \n\r", resultBuff[resultIdx]);

        /* Increment result idx */
        resultIdx = (resultIdx + 1) % (sizeof(resultBuff) / sizeof(resultBuff[0]));

        /* Clear interrupt source */
        Cy_Adc_Channel_ClearInterruptStatus(&CY_ADC_POT_MACRO->CH[ADC_LOGICAL_CHANNEL], &intrSource);
    }
    else
    {
        // Unexpected interrupt
        printf("Falscher ADC0 Interrupt\n\r");
        CY_ASSERT(false);
    }
}
